from .provider.authorization_server import AuthorizationServer
from .provider.resource_protector import ResourceProtector
from .sqla_oauth2 import (
    OAuth2AuthorizationCodeMixin,
    OAuth2ClientMixin,
    OAuth2TokenMixin,
    create_bearer_token_validator,
    create_query_client_func,
    create_revocation_endpoint,
    create_save_token_func,
)



from typing import Dict, List, Optional, Union

from pydantic import BaseSettings


class OAuthSetting(BaseSettings):
    OAUTH2_SCOPES_SUPPORTED: List[str] = ['vuong-open-id']
    OAUTH2_ERROR_URIS: List[str] = ['oauth/error']

    """
    OAUTH2_ACCESS_TOKEN_GENERATOR: Boolean or import string, default is True.
    
    Here are some examples of the token generator::

        OAUTH2_ACCESS_TOKEN_GENERATOR = 'your_project.generators.gen_token'

        # and in module `your_project.generators`, you can define:

        def gen_token(client, grant_type, user, scope):
            # generate token according to these parameters
            token = create_random_token()
            return f'{client.id}-{user.id}-{token}'
    """
    OAUTH2_ACCESS_TOKEN_GENERATOR: Union[bool, str] = True

    """
    OAUTH2_REFRESH_TOKEN_GENERATOR: Boolean or import string, default is False.
    By default, it will not generate ``refresh_token``, which can be turned on by
        configure ``OAUTH2_REFRESH_TOKEN_GENERATOR``.
    """
    OAUTH2_REFRESH_TOKEN_GENERATOR: Union[bool, str] = False

    """
    OAUTH2_TOKEN_EXPIRES_IN: Dict or import string, default is None.
    Here is an example of ``OAUTH2_TOKEN_EXPIRES_IN``::

        OAUTH2_TOKEN_EXPIRES_IN = {
            'authorization_code': 864000,
            'urn:ietf:params:oauth:grant-type:jwt-bearer': 3600,
        }
    """
    OAUTH2_TOKEN_EXPIRES_IN: Optional[Union[Dict, str]] = None



from typing import Type

from .rfc6749 import OAuth2Request


async def create_oauth_request(request, request_cls: Type[OAuth2Request]):
    if isinstance(request, request_cls):
        return request

    oauth_request = request_cls(request)
    await oauth_request.prepare_data()
    return oauth_request



name = 'FastAPI-OAuth'
version = '0.1.0'
author = 'Do Quoc Vuong <vuongtlt13@gmail.com>'
homepage = 'https://github.com/vuongtlt13/fastapi-oauth'
DEFAULT_USER_AGENT = '{}/{} (+{})'.format(name, version, homepage)

DEFAULT_JSON_HEADERS = [
    ('Content-Type', 'application/json'),
    ('Cache-Control', 'no-store'),
    ('Pragma', 'no-cache'),
]

ACCESS_TOKEN_LENGTH = 42
REFRESH_TOKEN_LENGTH = 48



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['fastapi_oauth',
 'fastapi_oauth.common',
 'fastapi_oauth.provider',
 'fastapi_oauth.rfc6749',
 'fastapi_oauth.rfc6749.grants',
 'fastapi_oauth.rfc6750',
 'fastapi_oauth.rfc7009',
 'fastapi_oauth.rfc7636',
 'fastapi_oauth.sqla_oauth2']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy[asyncio]>=1.4.42,<2.0.0',
 'Werkzeug>=2.2.2,<3.0.0',
 'fastapi>=0.85.1,<0.86.0',
 'pydantic[dotenv]>=1.10.2,<2.0.0']

setup_kwargs = {
    'name': 'fastapi-oauth',
    'version': '0.1.1',
    'description': 'OAuth2 Provider in FastAPI',
    'long_description': 'None',
    'author': 'Đỗ Quốc Vương',
    'author_email': 'vuongtlt13@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import time

from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession


def create_query_client_func(client_model):
    """Create an ``query_client`` function that can be used in authorization
    server.

    :param client_model: Client model class
    """

    async def query_client(client_id, session: AsyncSession):
        q = select(client_model).filter_by(client_id=client_id)
        return (await session.scalars(q)).first()

    return query_client


def create_save_token_func(token_model):
    """Create an ``save_token`` function that can be used in authorization
    server.

    :param token_model: Token model class
    """

    async def save_token(token, request, session: AsyncSession):
        if request.user:
            user_id = request.user.get_user_id()
        else:
            user_id = None
        client = request.client
        item = token_model(
            client_id=client.client_id,
            user_id=user_id,
            **token
        )
        session.add(item)
        await session.commit()

    return save_token


def create_query_token_func(token_model):
    """Create an ``query_token`` function for revocation, introspection
    token endpoints.

    :param token_model: Token model class
    """

    async def query_token(token, token_type_hint, session: AsyncSession):
        q = select(token_model)
        if token_type_hint == 'access_token':
            return (await session.scalars(q.filter_by(access_token=token))).first()
        elif token_type_hint == 'refresh_token':
            return (await session.scalars(q.filter_by(refresh_token=token))).first()
        # without token_type_hint
        item = (await session.scalars(q.filter_by(access_token=token))).first()
        if item:
            return item
        return (await session.scalars(q.filter_by(refresh_token=token))).first()

    return query_token


def create_revocation_endpoint(token_model):
    """Create a revocation endpoint class with SQLAlchemy session
    and token model.

    :param token_model: Token model class
    """
    query_token = create_query_token_func(token_model)

    from ..rfc7009 import RevocationEndpoint

    class _RevocationEndpoint(RevocationEndpoint):
        async def query_token(self, token, token_type_hint, session: AsyncSession):
            return await query_token(token, token_type_hint, session)

        async def revoke_token(self, token, request, session: AsyncSession):
            now = int(time.time())
            hint = request.form.get('token_type_hint')
            token.access_token_revoked_at = now
            if hint != 'access_token':
                token.refresh_token_revoked_at = now
            session.add(token)
            await session.commit()

    return _RevocationEndpoint


def create_bearer_token_validator(token_model):
    """Create a bearer token validator class with SQLAlchemy session
    and token model.

    :param token_model: Token model class
    """
    from ..rfc6750 import BearerTokenValidator

    class _BearerTokenValidator(BearerTokenValidator):
        async def authenticate_token(self, token_string, session: AsyncSession):
            q = select(token_model)
            return (await session.scalars(q.filter_by(access_token=token_string))).first()

        def request_invalid(self, request):
            return False

        def token_revoked(self, token):
            return token.revoked

    return _BearerTokenValidator
